@@ -7,6 +7,7 @@ import re, copy
 from twisted.internet import reactor, defer
 from twisted.web.resource import Resource
 from twisted.web.server import Site, NOT_DONE_YET
+from twisted.web.http import parse_qs
 from collections import defaultdict
 from enums import MediaType
 from corepost.enums import Http
@@ -99,6 +100,7 @@ class CorePost(Resource):
         '''
         Constructor
         '''
+        Resource.__init__(self)
         self.__urls = defaultdict(dict)
         self.__cachedUrls = defaultdict(dict)
         self.__methods = {}
@@ -140,27 +142,35 @@ class CorePost(Resource):
     def __renderUrl(self,request):
         """Finds the appropriate router and dispatches the request to the registered function"""
         # see if already cached
+        path = '/' + '/'.join(request.postpath)
+        
         urlrouter, pathargs = None, None
-        if request.path in self.__cachedUrls[request.method]:
-            cachedUrl = self.__cachedUrls[request.method][request.path]
+        if path in self.__cachedUrls[request.method]:
+            cachedUrl = self.__cachedUrls[request.method][path]
             urlrouter,pathargs = cachedUrl.router, cachedUrl.args 
         else:
             # first time this URL is called
             for router in self.__urls[request.method].values():
-                args = router.getArguments(request.path)
+                args = router.getArguments(path)
                 if args != None:
                     if router.cache:
-                        self.__cachedUrls[request.method][request.path] = CachedUrl(router, args)
+                        self.__cachedUrls[request.method][path] = CachedUrl(router, args)
                     urlrouter,pathargs = router,args
                     
         #actual call
         if urlrouter != None and pathargs != None:
             allargs = copy.deepcopy(pathargs)
+            # handler for weird Twisted logic where PUT does not get form params
+            # see: http://twistedmatrix.com/pipermail/twisted-web/2007-March/003338.html
+            requestargs = request.args
+            if request.method == Http.PUT:
+                requestargs = parse_qs(request.content.read(), 1)
+
             #merge form args
-            for arg in request.args.keys():
+            for arg in requestargs.keys():
                 # maintain first instance of an argument always
                 if arg not in allargs:
-                    allargs[arg] = request.args[arg][0]
+                    allargs[arg] = requestargs[arg][0]
                     
             # if POST/PUT, check if we need to automatically parse JSON
             # TODO
